#!/usr/bin/perl
# CONFIG: Change the first line to indicate the Perl binary on your system.
#          Try "which perl", "whereis perl", or "find / -name perl -print"

############################################################## -*- Perl -*- ###
#                                                                          
#     AccessWatch v2.0 - Traffic analysis for World Wide Web sites          
#      Copyright 1994-1998 David G. Maher. All rights reserved.          
#      http://accesswatch.com/                                              
#                                                                           
#     Contact Information:                                                  
#        AccessWatch                                                          
#        PO Box 183, Fairfax, VA 22030-0183 USA                               
#                                                                             
#     Please direct comments to:                                              
#        http://accesswatch.com/comments.html                                 
#                                                                             
#     Please read the included license agreement (readme.txt) before          
#      using this program. Your use of this software indicates your           
#      acceptance of the license agreement and warranty.                      
#                                                                             
#     If you wish to modify this code, mail changes@accesswatch.com.
#     We don't have a problem with modifications by registered users,
#     but redistribution is prohibited. Just write us first.
#                                                                             
#  Credits and thanks for great ideas:
#
#   A huge thank you to Paul Blackman <ictinus@lake.canberra.edu.au> for      
#   his work on hourly server statistics, including a table of hourly       
#   accesses/hour.                                                          
#                                                                           
#   Thanks to Jeff Boulter <jeff@boulter.com> for access error        
#   exclusion code, referer log parsing tips, and for his many suggestions  
#   and debugging help.                                                     
#                                                                           
#   Many thanks for suggestions and code tweaks:                              
#
#      Chris Brown     <cwb3@ra.msstate.edu>                                
#      Ron Gery        <rong@halcyon.com>                                   
#      Lars Bo Eriksen <lb@iesd.auc.dk>                                     
#      Tieshy Lin      <ted@iis.sinica.edu.tw>                              
#      Chris Derossi   <derossi@genmagic.com>                               
#      Arthur Hagen    <art@broomstick.com>                                 
#      Eric Ruck       <eruck@ahoynet.com>                                  
#                                                                           
###############################################################################

###############################################################################
########     NO MODIFICATIONS MAY BE MADE TO THE FOLLOWING CODE        ########
########            WITHOUT THE CONSENT OF THE AUTHOR                  ########
###############################################################################

($base=$0) =~ s/[A-z0-9,\.,\-]*$//;  # if you have trouble, define $base as
                                     #  the full directory pathname that 
                                     #  contains the accesswatch script.

require 'timelocal.pl' 
    || die "Error: Trouble locating timelocal.pl "
             . "- contact your system administrator $!\n";

require $base.'lib/global.lib'
    || die "Error: Global file not found $!\n"; 

require $base.'lib/db.lib' 
    || die "Error: Database access file not found $!\n"; 

require $base.'lib/time.lib' 
    || die "Error: Time library file not found $!\n"; 

##############################################################################

#-----------------------------------------------------------------------------#
#  AccessWatch function - Main
#    Purpose  : Coordinates everything else...
#-----------------------------------------------------------------------------#

MAIN: {

    my($beginS, $endS, $timeperiod);

    if ($opt_s) {
	($beginS, $endS) = &ReturnBeginEndMonth($opt_s);	
	$timeperiod = "$opt_s";
    }
    elsif ($opt_w) {
	($beginS, $endS) = ($weekS, $todayS);
	$timeperiod = "the last week";
    }
    elsif ($opt_m) {
	($beginS, $endS) = ($monthS, $todayS);
	$timeperiod = "the last month";
    }
    elsif ($opt_y) {
	($beginS, $endS) = ($yearS, $todayS);
	$timeperiod = "the last year";
    }
    elsif ($opt_d) {
	($beginS, $endS) = ($yestrS, $todayS);
	$timeperiod = "the last day";
    }
    else {
	($beginS, $endS) = (0, time);
	$timeperiod = "the entire log";
    }

    &removeFiles;

    print "AccessWatch v$version starting, parsing $timeperiod.\n"
	if $verbose;

    open (LOG, "<$accessLog") 
	|| die "Access Log Error: Couldn't open $accessLog $!\n";
    
    my($firstDateFoundS, $lastDateFoundS) = &ProcessAccessLog($beginS, $endS);
    
    close (LOG);

    print "Log file parsing complete.\n" 
	if $verbose;

    exit 0;
}


#-----------------------------------------------------------------------------#
#  AccessWatch function - ProcessAccessLog
#    Purpose  : Scans through access log and picks out the appropriate accesses
#-----------------------------------------------------------------------------#
sub ProcessAccessLog {

#LOG:$remote, $d1, $d2, $date, $tz, $method, $page, $protocol, $protnum1, $size

    my($beginTime, $endTime) = ($_[0], $_[1]);

    my($firstDateFoundS) = &FastSearch($beginTime);

    if (!$firstDateFoundS) {
	print "Problems with the log file - search date not found.\n";
	exit 1;
    }

    my(@hourHosts, $currentHour, $lastDateFoundS);

    my($remote, $dash1, $dash2, $date, $tz, 
	  $method, $page, $protocol, $protnum1, $size, $ref, $agent);

    my($hour, $min, $sec, $lday, $pday, $lmonth, $lyear, $datecode);

    my($lineTimeLF, $futtimes);
    my($beginTimeLF) = &FormatLogTime($firstDateFoundS);
    my($endTimeLF) = &FormatLogTime($endTime);

    print "Begin time: $beginTimeLF\tEnd time  : $endTimeLF\n" if $verbose;
    
    $| = 1; 
    
    print "Parsing access log..." if $verbose;

    while (<LOG>) {		
#	if ($verbose && $periodStats{'serverCount'} % 500 == 1) {
#	    print "." if $verbose;
#	    if (open(DEBUG, ">$base/progress.log")) {
#		print DEBUG "$lineTimeLF\n";
#		close DEBUG;
#	    }	   
#	}
	
	$lineTimeLF = $_;
	$lineTimeLF =~ s/^[^\]]+\[([^\]]+).*$/$1/goi;
	
	if (&CompareLogTime($beginTimeLF, $lineTimeLF) < 0) {
	    next;
	}
	elsif (&CompareLogTime($endTimeLF, $lineTimeLF) > 0) {
#	    print "Found last time End = $endTimeLF Line = $lineTimeLF\n";
	    $futtimes++;

	    if ($futtimes > 400) {
		last;
	    }
	    next;
	}
	
        $periodStats{'serverCount'}++;
	
	next unless 
(/^(\S+) \S+ \S+ \[(\S+):(\S+):(\S+):(\S+) \S+\] \S+ (\S+) \S+ (\S+) (\S+)/o);
	
	($remote, $date, $page, $protnum1, $size) = ($1, $2, $6, $7, $8);
        ($hour, $min, $sec) = ($3, $4, $5);
	($lday, $lmonth, $lyear) = split('/', $date);



        $newdatecode = sprintf("%04d%02d%02d%02d", 
				$lyear, $monthTable{$lmonth}, $lday, $hour);

        if ($newdatecode ne $datecode) {
	    print ".$newdatecode." if ($pday ne $lday && $verbose);
#	    print "Old: $datecode New: $newdatecode\n";

	    foreach $oldcode (sort keys %AWdata) {
		if (abs($oldcode - $datecode) > 2 && $AWdata{$oldcode}) { 
                    # data more than 3 hours old - takes care 
                    # of log file caching by busy servers.
#		    print "CACHE Writing Stale $oldcode\n";
		    &writeFile($oldcode);
		    delete $AWdata{$oldcode};
		}
	    }

	    if (! %{ $AWdata{$newdatecode} }) {
#		print "CACHE Reading New $newdatecode\n";
		&readFile($newdatecode);
	    }
	    $datecode = $newdatecode;
        }
        
        if (m|\"([^\"]+)\" \"([^\"]+)\"$|o) {

	    ($ref, $agent) = ($1, $2);
	    if ($ref eq '' || $ref eq '-' || $ref =~ /$orgdomain/i) {
		undef $ref;
	    }
	    if ($agent eq '' || $agent eq '-') {
		undef $agent;
	    }
	}
        else {
	    undef $ref;
	    undef $agent;
        }
	
#        print ".$lmonth/$lday/$lyear." if ($pday ne $lday && $verbose);
        $pday = $lday;
	
        $page = &FilterURL($page); 
	
        $remote .= ".$orgdomain" if ($remote !~ /\./o);
        #takes care of those internal accesses that do not get fully 
        # qualified in the log name -> name.orgname.ext
	
	next unless $page =~ /$includeURL/io 
           && !($remote =~ /$excludeHosts/io && $excludeHosts ne "");
	
	# Compress proxies
	$remote =~ tr/[A-Z]/[a-z]/;
	$remote =~ s/.*.proxy.aol.com/www.proxy.aol.com/oi;
	$remote 
            =~ s/piweb[a-z][0-9][a-z]-ext.prodigy.com/piweb-ext.prodigy.com/oi;

        $AWdata{$datecode}{'codes'}{$protnum1}++;
        $AWdata{$datecode}{'hosts'}{$remote}++;
	$AWdata{$datecode}{'pages'}{$page}++;
	$AWdata{$datecode}{'filesize'}{$page} += $size if ($size =~ /^\d+$/);
	$AWdata{$datecode}{'browsers'}{$agent}++ if $agent;
	$AWdata{$datecode}{'references'}{"$ref~~~$page"}++ if $ref;
    }				

    # flush the cache
    foreach $hour (sort keys %AWdata) {
#        print "CACHE Flushing $hour\n";
        &writeFile($hour);
    }

    print " done.\n" if $verbose;
   
    $| = 1; 
    
    return ($firstDateFoundS, 
	    &ReturnSecondsFromLogDate("$date:$hour:$min:$sec"));
}


#-----------------------------------------------------------------------------#
#  AccessWatch function - FastSearch
#    Purpose  : Scans quickly from the bottom of the access log to start
#                   of the given day.
#-----------------------------------------------------------------------------#

sub FastSearch {

    my($searchTime) = $_[0];

    if ($verbose) {
	$| = 1; print "Scanning for start of current log period..."; $| = 0;
    }

    if ($searchTime == 0) {
	seek(LOG, 0, 0); <LOG>; 
	$_ = <LOG>;
	seek(LOG, 0, 0);
	<LOG>;
	print " done.\n" if $verbose;
	return &ReturnSecondsFromLogDate($_);
    }

    my($searchTimeLF) = &FormatLogTime($searchTime);
    my($lineTimeLF) = "00/Jan/0000:00:00:00";

    my($avgDayGrowth) = 10000 * $hour + 
	(10000 * 24 * ($todayS - $searchTime)/(24*60*60)) + 1000;
    my($jumpsize) = $avgDayGrowth/5;

    my($logsize) = (-s LOG);
    my($logoffset, $logPosDiff, $jumps) = 0;

    $logoffset = sprintf("%d", $logsize - $avgDayGrowth)
	if ($logsize > $avgDayGrowth);

    seek (LOG, $logoffset, 0);
    <LOG>;

    search: while (<LOG>) {

	last search if ($logoffset < 0);

	$lineTimeLF = $_;
	$lineTimeLF =~ s/^[^\]]+\[([^\]]+).*$/$1/gi;

	$logPosDiff = &CompareLogTime($searchTimeLF, $lineTimeLF);

	if ($logPosDiff > 0) {
	    $logoffset = $logoffset - $jumpsize;
	}
	elsif ($logPosDiff < 0) {
	    forwardScan: while (<LOG>) { 
		$lineTimeLF = $_;
		$lineTimeLF =~ s/^[^\]]+\[([^\]]+).*$/$1/gi;     
		$logPosDiff = &CompareLogTime($searchTimeLF, $lineTimeLF);
		last forwardScan if ($logPosDiff >= 0);
	    }
	    last search;
	}
	else { last search; }

	$jumps++;

	if ($verbose && $jumps % 100 == 1) {
	    $| = 1;
	    print ".";
	    $| = 0;
	}

	seek (LOG, $logoffset, 0);
	<LOG>;
    }
    print " done.\n" if $verbose;
    
    return &ReturnSecondsFromLogDate($_);
}


#-----------------------------------------------------------------------------#
#  AccessWatch function - FilterURL
#    Purpose  : Takes a URL as an argument and decodes/trims it.
#-----------------------------------------------------------------------------#

sub FilterURL {

    my($url) = $_[0];

    $url =~ tr/+/ /;
    $url =~ s/([^\?]+)\?.*/$1/o;        #strip last '?' from GET requests
    $url =~ s/([^\.]+)[\.]+$/$1/o;      #strip last '.'
    $url =~ s/$defaultPage$//o;
    $url =~ s/([^\/]+)[\/]+$/$1/o;      #strip last '/'
    $url =~ s/([^\#]+)\#.*/$1/o;        #strip last '#' from label access
    $url =~ s/(\/)[\/]+/$1/og;

    $url =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/ego;

    $url = "\/" if ($url eq "");
    return $url;
}


__END__


