/*
 * Bouncing balls - each ball is a different thread with a different
 * time to sleep.
 * Adds pause and continue buttons.
 */

import java.awt.*;
import java.awt.event.*;

public class BouncePause extends Frame implements ActionListener
{
   private Canvas canvas;
   private int maxthreads = 100;
   private Thread[] threadarray = new Thread[maxthreads];
   private int lastthread = -1;

   public BouncePause()
   {  setLayout(new BorderLayout());
      setTitle("BouncePause");

      canvas = new Canvas();
      add(canvas, BorderLayout.CENTER);

      Panel p = new Panel();
      Button b1 = new Button("Start");
      b1.addActionListener(this);
      p.add(b1);
      Button b3 = new Button("Pause");
      b3.addActionListener(this);
      p.add(b3);
      Button b4 = new Button("Continue");
      b4.addActionListener(this);
      p.add(b4);
      Button b2 = new Button("Close");
      b2.addActionListener(this);
      p.add(b2);
      add(p, BorderLayout.SOUTH);
   }
   
   public void actionPerformed(ActionEvent evt)
   {  if (evt.getActionCommand().equals("Start"))
      {  if (lastthread < (maxthreads - 1))
	 { lastthread++;
           BallTimes b = new BallTimes(canvas);
	   threadarray[lastthread] = b;
           b.start();
         }
      }
      else if (evt.getActionCommand().equals("Pause"))
	{ for (int i = 0; i < lastthread; i++)
	     threadarray[i].suspend();
        }
      else if (evt.getActionCommand().equals("Continue"))
	{ for (int i = 0; i < lastthread; i++)
	     threadarray[i].resume();
        }
      else if (evt.getActionCommand().equals("Close"))
         System.exit(0);
   }
 
   public static void main(String[] args)
   {  Frame f = new BouncePause();
      f.setSize(400, 300);
      f.setVisible(true);  
   }
   
}

class BallTimes extends Thread
{  
   private Canvas box;
   private static final int XSIZE = 10;
   private static final int YSIZE = 10;
   private int x = 0;
   private int y = 0;
   private int dx = 2;
   private int dy = 2;   
   private int sleeptime;

   public BallTimes(Canvas c) 
   { box = c;
     sleeptime = ((int)(Math.random() * 20)) + 20;
     System.out.print(sleeptime+" ");
     
   }

   public void draw()
   {  Graphics g = box.getGraphics();
      g.fillOval(x, y, XSIZE, YSIZE);
      g.dispose();
   }
   
   public void move()
   {  Graphics g = box.getGraphics();
      g.setXORMode(box.getBackground());
      g.fillOval(x, y, XSIZE, YSIZE);
      x += dx;
      y += dy;
      Dimension d = box.getSize();
      if (x < 0) { x = 0; dx = -dx; }
      if (x + XSIZE >= d.width) { x = d.width - XSIZE; dx = -dx; }
      if (y < 0) { y = 0; dy = -dy; }
      if (y + YSIZE >= d.height) { y = d.height - YSIZE; dy = -dy; }
      g.fillOval(x, y, XSIZE, YSIZE);
      g.dispose();
   }
   
   public void run()
   {  draw();
      while (true)
      {  move();
         try { Thread.sleep(sleeptime); } catch(InterruptedException e) {}
      }
   }
}

