/* 
 * This applet demonstrates the following classes:  Frame, MenuBar, 
 * Menu, MenuItem, CheckboxMenuItem, MenuShortcut and Dialog.
 *
 * Note that MenuItems generate ActionEvents, while CheckboxMenuItems 
 * generate ItemEvents.
 *
 */

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.StringTokenizer;

public class Netscape {

  public static void main( String[] args ) {
    new NewWebBrowser();
  }
  
}

// public class Netscape extends java.applet.Applet {
// 
//   public void init() {
//     new NewWebBrowser();
//   }
//   
// }

// Simulate a Netscape web browser window:
class NewWebBrowser extends Frame {

  protected static int num_windows = 0;
  
  MenuBar menubar;
  TextArea textarea;
    
//   char ellipsis = '\u2026';  // Why doesn't this work?
  String ellipsis = "...";

  public NewWebBrowser() {
  
    // Invoke the constructor of the superclass:
    super( "Web Browser" );
    
    //setFont( new Font( "SansSerif", Font.BOLD, 12 ) );
    setFont( new Font( "Serif", Font.ITALIC, 18 ) );
    num_windows++;
    setSize( 700, 800 );
    
    textarea = new TextArea();
    add( textarea, BorderLayout.CENTER );
    
    menubar = new MenuBar();
    
    menubar.add( new FileMenu() );
    menubar.add( new EditMenu() );
    menubar.add( new ViewMenu() );
    menubar.add( new GoMenu() );
    menubar.add( new BookmarksMenu() );
    menubar.add( new OptionsMenu() );
    menubar.add( new DirectoryMenu() );
    menubar.add( new WindowMenu() );
    menubar.setHelpMenu( new HelpMenu() );  // the Help menu is special
    
    setMenuBar( menubar );
    setVisible( true );
  }
  
  class FileMenu extends Menu implements ActionListener {
  
    // Menu item names:
    MenuItem newWebBrowserItem,
             newMailMessageItem,
             mailDocumentItem,
             openLocationItem,
             openFileItem,
             saveAsItem,
             uploadFileItem,
             printItem,
             closeItem,
             exitItem;
    
    // Menu constructor:
    public FileMenu() {
    
      // Invoke the constructor of the superclass:
      super( "File" );

//       setFont( new Font( "SansSerif", Font.BOLD, 14 ) );
    
      // Menu items with shortcuts:
      newWebBrowserItem  
        = new MenuItem( "New Web Browser", 
                        new MenuShortcut( KeyEvent.VK_N ) );
      newMailMessageItem 
        = new MenuItem( "New Mail Message", 
                        new MenuShortcut( KeyEvent.VK_M ) );
      mailDocumentItem   
        = new MenuItem( "Mail Document" );
      openLocationItem   
        = new MenuItem( "Open Location" + ellipsis, 
                        new MenuShortcut( KeyEvent.VK_L ) );
      openFileItem       
        = new MenuItem( "Open File" + ellipsis, 
                        new MenuShortcut( KeyEvent.VK_O ) );
      saveAsItem         
        = new MenuItem( "Save As" + ellipsis, 
                        new MenuShortcut( KeyEvent.VK_S ) );
      uploadFileItem     
        = new MenuItem( "Upload File" + ellipsis );
      printItem          
        = new MenuItem( "Print" + ellipsis, 
                        new MenuShortcut( KeyEvent.VK_P ) );
      closeItem          
        = new MenuItem( "Close", 
                        new MenuShortcut( KeyEvent.VK_W ) );
      exitItem           
        = new MenuItem( "Exit", 
                        new MenuShortcut( KeyEvent.VK_Q ) );
      
      // Build the menu:
      add( newWebBrowserItem );
      add( newMailMessageItem );
      add( mailDocumentItem );
      addSeparator();
      add( openLocationItem );
      add( openFileItem );
      add( saveAsItem );
      add( uploadFileItem );
      addSeparator();
      add( printItem );
      addSeparator();
      add( closeItem );
      add( exitItem );
      
      // Disable menu item(s):
      uploadFileItem.setEnabled( false );

      // Listen for ActionEvents generated by menu items:
      newWebBrowserItem.addActionListener( this );
      openFileItem.addActionListener( this );
      exitItem.addActionListener( this );
    }
  
    // ActionEvent handler:
    public void actionPerformed( ActionEvent e ) {
      if ( e.getSource() == newWebBrowserItem ) {
        new NewWebBrowser();
      } else if ( e.getSource() == openFileItem ) {
//         OpenFileDialog d = new OpenFileDialog( NewWebBrowser.this );
        FileDialog d = new FileDialog( NewWebBrowser.this, 
                                       "Open File", 
                                       FileDialog.LOAD );
	d.setFont( new Font( "Serif", Font.ITALIC, 18 ) );
        d.setDirectory( "." );
        d.setVisible( true );
        String filename = d.getFile();
        if ( filename != null ) {
          String dirname = d.getDirectory();
          File fileObj = new File( dirname + filename );
          try {
            RandomAccessFile file = new RandomAccessFile( fileObj, "r" );
            StringBuffer buf = new StringBuffer();
            String text;
            while ( ( text = file.readLine() ) != null ) {
              buf.append( text + "\n" );
            }
            textarea.setText( buf.toString() );
          } catch ( IOException e2 ) {
            System.out.println( e2.toString() );
          }
        }
      } else if ( e.getSource() == exitItem ) {
        if ( --num_windows == 0 ) {
          System.exit(0);
        } else {
          setVisible( false );
          dispose();
        }
      }
    }
    
  }

  class EditMenu extends Menu implements ActionListener {
  
    // Menu item names:
    MenuItem undoItem,
             cutItem,
             copyItem,
             pasteItem,
             findItem,
             findAgainItem;
    
    // Menu constructor:
    public EditMenu() {
    
      // Invoke the constructor of the superclass:
      super( "Edit" );

//       setFont( new Font( "SansSerif", Font.BOLD, 14 ) );
    
      // Menu items with shortcuts:
      undoItem  
        = new MenuItem( "Undo", 
                        new MenuShortcut( KeyEvent.VK_Z ) );
      cutItem 
        = new MenuItem( "Cut", 
                        new MenuShortcut( KeyEvent.VK_X ) );
      copyItem 
        = new MenuItem( "Copy", 
                        new MenuShortcut( KeyEvent.VK_C ) );
      pasteItem 
        = new MenuItem( "Paste", 
                        new MenuShortcut( KeyEvent.VK_V ) );
      findItem   
        = new MenuItem( "Find" + ellipsis, 
                        new MenuShortcut( KeyEvent.VK_F ) );
      findAgainItem   
        = new MenuItem( "Find Again", 
                        new MenuShortcut( KeyEvent.VK_G ) );
      
      // Build the menu:
      add( undoItem );
      addSeparator();
      add( cutItem );
      add( copyItem );
      add( pasteItem );
      addSeparator();
      add( findItem );
      add( findAgainItem );
  
      // Disable menu item(s):
      
      // Listen for ActionEvents generated by menu items:

    }
  
    // ActionEvent handler:
    public void actionPerformed( ActionEvent e ) {
    }
    
  }

  class ViewMenu extends Menu implements ActionListener {
  
    // Menu item names:
    
    // Menu constructor:
    public ViewMenu() {
    
      // Invoke the constructor of the superclass:
      super( "View" );

//       setFont( new Font( "SansSerif", Font.BOLD, 14 ) );
    
      // Menu items with shortcuts:
      
      // Build the menu:
  
      // Disable menu item(s):
      
      // Listen for ActionEvents generated by menu items:

    }
  
    // ActionEvent handler:
    public void actionPerformed( ActionEvent e ) {
    }
    
  }

  class GoMenu extends Menu implements ActionListener {
  
    // Menu item names:
    
    // Menu constructor:
    public GoMenu() {
    
      // Invoke the constructor of the superclass:
      super( "Go" );

//       setFont( new Font( "SansSerif", Font.BOLD, 14 ) );
    
      // Menu items with shortcuts:
      
      // Build the menu:
  
      // Disable menu item(s):
      
      // Listen for ActionEvents generated by menu items:

    }
  
    // ActionEvent handler:
    public void actionPerformed( ActionEvent e ) {
    }
    
  }

  class BookmarksMenu extends Menu implements ActionListener {
  
    // Menu item names:
    
    // Menu constructor:
    public BookmarksMenu() {
    
      // Invoke the constructor of the superclass:
      super( "Bookmarks" );

//       setFont( new Font( "SansSerif", Font.BOLD, 14 ) );
    
      // Menu items with shortcuts:
      
      // Build the menu:
  
      // Disable menu item(s):
      
      // Listen for ActionEvents generated by menu items:

    }
  
    // ActionEvent handler:
    public void actionPerformed( ActionEvent e ) {
    }
    
  }

  class OptionsMenu extends Menu implements ActionListener {
  
    // Menu item names:
    MenuItem generalPrefs,
             mailAndNewsPrefs,
             networkPrefs,
             securityPrefs;
    
    // Checkable menu item names:
    CheckboxMenuItem showMenubarItem,
                     showToolbarItem,
                     showLocationItem,
                     showDirectoryButtonsItem,
                     showJavaConsoleItem;
    
    // Menu constructor:
    public OptionsMenu() {
    
      // Invoke the constructor of the superclass:
      super( "Options" );

//       setFont( new Font( "SansSerif", Font.BOLD, 14 ) );
    
      // Menu items:
      generalPrefs = new MenuItem( "General Preferences" + ellipsis );
      mailAndNewsPrefs = new MenuItem( "Mail and News Preferences" + ellipsis );
      networkPrefs = new MenuItem( "Network Preferences" + ellipsis );
      securityPrefs = new MenuItem( "Security Preferences" + ellipsis );
      
      // Checkable menu items:
      showMenubarItem = new CheckboxMenuItem( "Show Menubar" );
      showMenubarItem.setState( true );  // checked by default
      showToolbarItem = new CheckboxMenuItem( "Show Toolbar" );
      showLocationItem = new CheckboxMenuItem( "Show Location" );
      showDirectoryButtonsItem = new CheckboxMenuItem( "Show Directory Buttons" );
      showJavaConsoleItem = new CheckboxMenuItem( "Show Java Console" );
      
      // A submenu of checkable menu items:
      Menu submenu = new Menu( "Document Encoding" );
      submenu.add( new CheckboxMenuItem( "Western (Latin-1)", true ) );
      submenu.addSeparator();
      submenu.add( new CheckboxMenuItem( "Central European (Latin-2)" ) );
      submenu.addSeparator();
      submenu.add( new CheckboxMenuItem( "Japanese (Auto-Detect)" ) );
      submenu.add( new CheckboxMenuItem( "Japanese (Shift-JIS)" ) );
      submenu.add( new CheckboxMenuItem( "Japanese (EUC-JP)" ) );
      // (many more such items omitted)

      // Build the menu:
      add( generalPrefs );
      add( mailAndNewsPrefs );
      add( networkPrefs );
      add( securityPrefs );
      addSeparator();
      add( showMenubarItem );
      add( showToolbarItem );
      add( showLocationItem );
      add( showDirectoryButtonsItem );
      add( showJavaConsoleItem );
      addSeparator();
      add( submenu );
  
      // Disable menu item(s):
      
      // Listen for ActionEvents generated by menu items:

    }
  
    // ActionEvent handler:
    public void actionPerformed( ActionEvent e ) {
    }
    
  }

  class DirectoryMenu extends Menu implements ActionListener {
  
    // Menu item names:
    
    // Menu constructor:
    public DirectoryMenu() {
    
      // Invoke the constructor of the superclass:
      super( "Directory" );

//       setFont( new Font( "SansSerif", Font.BOLD, 14 ) );
    
      // Menu items with shortcuts:
      
      // Build the menu:
  
      // Disable menu item(s):
      
      // Listen for ActionEvents generated by menu items:

    }
  
    // ActionEvent handler:
    public void actionPerformed( ActionEvent e ) {
    }
    
  }

  class WindowMenu extends Menu implements ActionListener {
  
    // Menu item names:
    
    // Menu constructor:
    public WindowMenu() {
    
      // Invoke the constructor of the superclass:
      super( "Window" );

//       setFont( new Font( "SansSerif", Font.BOLD, 14 ) );
    
      // Menu items with shortcuts:
      
      // Build the menu:
  
      // Disable menu item(s):
      
      // Listen for ActionEvents generated by menu items:

    }
  
    // ActionEvent handler:
    public void actionPerformed( ActionEvent e ) {
    }
    
  }

  class HelpMenu extends Menu implements ActionListener {
  
    // Menu item names:
    
    // Menu constructor:
    public HelpMenu() {
    
      // Invoke the constructor of the superclass:
      super( "Help" );

//       setFont( new Font( "SansSerif", Font.BOLD, 14 ) );
    
      // Menu items with shortcuts:
      
      // Build the menu:
      add( new MenuItem( "About Netscape" ) );
      add( new MenuItem( "About Plug-ins" ) );
      add( new MenuItem( "Registration Information" ) );
      add( new MenuItem( "Software" ) );
      addSeparator();
      add( new MenuItem( "Handbook" ) );
  
      // Disable menu item(s):
      
      // Listen for ActionEvents generated by menu items:

    }
  
    // ActionEvent handler:
    public void actionPerformed( ActionEvent e ) {
    }
    
  }

}
  
class OpenFileDialog extends Dialog 
                     implements ItemListener, ActionListener {

  private Choice choice = new Choice();
  private List list = new List( 5, false );
  private Button cancel, open;
  
  private String path;
  String file;
  
  public OpenFileDialog( Frame parentFrame ) {
    // Invoke the constructor of the superclass:
    super( parentFrame, "Open File", true );
    
    file = null;
    
    setSize( 300, 300 );
    setResizable( false );
    
    String currentDir = System.getProperty( "user.dir" );
    buildChoice( currentDir );
    buildList( currentDir );
    
    Panel listPanel = new Panel();
    listPanel.setLayout( new BorderLayout() );
    listPanel.add( choice, BorderLayout.NORTH );
    listPanel.add( list, BorderLayout.CENTER );
    
    Panel buttonPanel = new Panel();
    buttonPanel.setLayout( new GridLayout( 2, 1, 30, 30 ) );
    cancel = new Button( "Cancel" );
    open = new Button( "Open" );
    cancel.setSize( 20, 50 );
    open.setSize( 20, 50 );
    buttonPanel.add( cancel );
    buttonPanel.add( open );
    
    add( listPanel, BorderLayout.CENTER );
    add( buttonPanel, BorderLayout.EAST );
    
    choice.addItemListener( this );
    list.addActionListener( this );
    cancel.addActionListener( this );
    open.addActionListener( this );
    
    setVisible( true );
  }
  
  public String getFile() { return file; }
  
  public void itemStateChanged( ItemEvent e ) {
    if ( e.getSource() == choice ) {
      int k = choice.getSelectedIndex();
//       for ( int i = 0; i < k; i++ ) {
//         choice.remove(0);
//       }
      path = choice.getSelectedItem();
      for ( int i = k + 1; i < choice.getItemCount(); i++ ) {
        path = choice.getItem(i) + File.separator + path;
      }
      path = File.separator + path;
      buildChoice( path );
      buildList( path );
    }
  }
    
  public void actionPerformed( ActionEvent e ) {
    if ( e.getSource() == cancel ) {
      setVisible( false );
      dispose();
    } else {
      String item = list.getSelectedItem();
      String tempPath = path + File.separator + item;
      File fileObj = new File( tempPath );
      if ( fileObj.isDirectory() ) {
        path = tempPath;
        buildChoice( path );
        buildList( path );
      } else if ( fileObj.isFile() ) {
        file = tempPath;
        setVisible( false );
        dispose();
      }
    }
  }
  
  public void buildChoice( String path ) {
    StringTokenizer tokens = new StringTokenizer( path );
    choice.setVisible( false );
    choice.removeAll();
    while ( tokens.hasMoreTokens() ) {
      choice.insert( tokens.nextToken( File.separator ), 0 );
    }
    choice.setVisible( true );
  }
    
  public void buildList( String path ) {
    File fileObject = new File( path );
    String item[] = fileObject.list();
    list.removeAll();
    for ( int i = 0; i < item.length; i++ ) {
      list.addItem( item[i] );
    }
  }
    
}
	     
