/* PictureTime.java - This example illustrates the hierarchical use
   of components in a grid layout.
   Author:  Meryem Ispirli  Revised:  Nancy McCracken 7/21/97
*/

import java.awt.*;
/*TANGO*/    // TANGO API classes for collaborative framework
/*TANGO*/    import interfaces.AppletBase;
/*TANGO*/    import messages.*;
/*TANGO*/    import consts.*;

/*TANGO*/ public class PictureTime extends AppletBase {
/*TANGO*/   boolean Master = true;   // to indicate master/slave status
 
/* instance variable to hold the subpanel component */  
  ControlPanel control;

/* instance variable to hold the canvas component */
  Canvas displaycanvas;


  public void init() {  
/*TANGO*/      register();   // TANGO registration, this is the first method called in init method
    
  /* using grid layout with gap of 15 points separating each component */
    setLayout(new GridLayout(1,2,15,15)); 
   
    displaycanvas = new Canvas();  /* creating the canvas component */
    displaycanvas.setBackground(Color.blue); 
    
  /* creating instance for PictureControl panel */
    control = new ControlPanel(this);

  /* adding the components to the panel */
    add(displaycanvas);  
    add(control);
  }
  
  public Insets insets() {
    return new Insets(20,0,20,0);
  }

  void changeCanvasColor(String category) {
    if (category.equals("Turkiye")) 
                displaycanvas.setBackground(Color.magenta);
    else if (category.equals("San Diego")) 
                displaycanvas.setBackground(Color.cyan);
    else if (category.equals("Syracuse")) 
                displaycanvas.setBackground(Color.pink);
    else displaycanvas.setBackground(Color.black);
  }
/*TANGO*/    // recieve events from Mater (Slave Only), this method is required
/*TANGO*/    public void receive (AppEventMessage em) {
/*TANGO*/      try {
/*TANGO*/          // by default, when a new instance joins, it gets an master/slave message
 /*TANGO*/           if (em.getType() == Const.CONTROL) {
/*TANGO*/                    CAppAnswerEventMessage msg =  (CAppAnswerEventMessage) em;
/*TANGO*/                    if (msg.getAnswerType() == CAConsts.ISMASTER) {
/*TANGO*/                            Master = msg.isMaster(); // update master/slave status
/*TANGO*/                            if (Master) System.out.println ("I am master");
/*TANGO*/                            else System.out.println ("I am slave");
/*TANGO*/                    }
/*TANGO*/                    return;
/*TANGO*/          }
/*TANGO*/
/*TANGO*/          // slave decides what to do upon the event received  from the master
/*TANGO*/          String r=new String(em.getData(),0,0,em.getLen()); // decode the message into a String
/*TANGO*/          changeCanvasColor(r); 
/*TANGO*/      }
/*TANGO*/      catch (Exception e) {
/*TANGO*/        e.printStackTrace();
/*TANGO*/      }
/*TANGO*/    }
}

/* this class is a subclass of Panel */

class ControlPanel extends Panel { 
/* instance variable to hold a pointer to calling class */
  PictureTime outerparent;

/* the three control buttons */
  Button t, d, s;   
  
  ControlPanel(PictureTime target) {
    this.outerparent = target;
    setLayout(new GridLayout(3,1,0,60));
    
    /* putting the buttons */
    setBackground(Color.lightGray);
    t=new Button("Turkiye");
    add(t);
    t.setForeground(Color.white);
    t.setBackground(Color.magenta);
    d=new Button("San Diego");
    add(d);
    d.setForeground(Color.blue);
    d.setBackground(Color.cyan);
    s=new Button("Syracuse");
    add(s);
    s.setForeground(Color.black);
    s.setBackground(Color.pink);
  }
  
  public boolean action(Event evt, Object arg) {
    /*TANGO*/   //   if (!this.outerparent.Master) return true;   // only master can act. If you want to run the
                                 // applet in Master-Master mode, comment out this line
   if (evt.target instanceof Button) {
     String str=(String)arg;
     this.outerparent.changeCanvasColor(str);
/*TANGO*/            /* Send events over TANGO */
/*TANGO*/              int len=str.length();
/*TANGO*/              // prepare message to be sent by TANGO runtime
/*TANGO*/              byte[] t=new byte[len];
/*TANGO*/              str.getBytes(0,len,t,0);
/*TANGO*/             AppEventMessage em=new AppEventMessage(Const.DATA,t);
/*TANGO*/             this.outerparent.send(em); 
   }
   return true;
  }
}










